@@ -81,8 +81,17 @@ line_not_blank(byte *s)
 private int
 read_binary(byte *p, unsigned int max)
 {
-	/* errno */
-	return fread(p, sizeof(byte), max, stdin);
+	if (feof(stdin)) {
+		exit(EXIT_SUCCESS);
+	}
+
+	size_t ret = fread(p, sizeof(byte), max, stdin);
+
+	if (ferror(stdin)) {
+		warn_exit("error in read_binary");
+	}
+
+	return ret;
 }
 
 private int
@@ -118,6 +127,9 @@ read_radix64(byte *p, unsigned int max)
 			done = YES;
 			return out;
 		}
+		if (c >= 128) {
+		  continue;
+		}
 		d = base256[c];
 		switch (d) {
 		case OOB:
